import 'package:flutter/material.dart';

import 'package:is_lock_screen2/is_lock_screen2.dart';
import 'package:zego_plugin_adapter/zego_plugin_adapter.dart';

import 'package:zego_zimkit/src/callkit/defines.dart';
import 'package:zego_zimkit/src/callkit/notification_manager.dart';
import 'package:zego_zimkit/src/callkit/variables.dart';
import 'package:zego_zimkit/src/services/core/core.dart';
import 'package:zego_zimkit/src/services/logger_service.dart';
import 'package:zego_zimkit/src/utils/share_pref.dart';

mixin ZIMKitCoreBackgroundMessage {
  bool _isAppInBackground = false;

  Future<void> initBackgroundMessage({
    ZegoZIMKitNotificationConfig? notificationConfig,
  }) async {
    ZIMKitLogger.info(
      'init background message, config:$notificationConfig',
    );

    ZegoPluginAdapter()
        .service()
        .registerMessageHandler(_onAppLifecycleStateChanged);

    ZIMKitCore.instance.db.messageArrivedNotifier
        .addListener(_onMessageArrived);

    await initNotification(
      notificationConfig: notificationConfig,
    );
  }

  Future<void> initNotification({
    ZegoZIMKitNotificationConfig? notificationConfig,
  }) async {
    if (!(notificationConfig?.supportBackgroundMessage ?? false)) {
      ZIMKitLogger.info(
        'background message, but not support background message',
      );

      return;
    }

    await setPreferenceString(
      serializationKeyHandlerPrivateInfo,
      ZimKitHandlerPrivateInfo(
        channelID: notificationConfig?.androidNotificationConfig?.channelID ??
            defaultZIMKitMessageChannelID,
        channelName:
            notificationConfig?.androidNotificationConfig?.channelName ??
                defaultZIMKitMessageChannelName,
        sound: notificationConfig?.androidNotificationConfig?.sound ?? '',
        icon: notificationConfig?.androidNotificationConfig?.icon ?? '',
        isVibrate:
            notificationConfig?.androidNotificationConfig?.vibrate ?? false,
      ).toJsonString(),
    );

    await ZegoZIMKitNotificationManager.instance.init(
      notificationConfig: notificationConfig,
    );
  }

  void uninitBackgroundMessage() {
    ZegoPluginAdapter()
        .service()
        .unregisterMessageHandler(_onAppLifecycleStateChanged);

    ZIMKitCore.instance.db.messageArrivedNotifier
        .removeListener(_onMessageArrived);
  }

  Future<void> _onAppLifecycleStateChanged(
      AppLifecycleState appLifecycleState) async {
    final isScreenLockEnabled = await isLockScreen() ?? false;
    _isAppInBackground = appLifecycleState != AppLifecycleState.resumed;
    if (isScreenLockEnabled) {
      _isAppInBackground = true;
    }

    ZIMKitLogger.info(
      'AppLifecycleStateChanged, state:$appLifecycleState, '
      'isAppInBackground:$_isAppInBackground, '
      'isScreenLockEnabled:$isScreenLockEnabled',
    );
  }

  Future<void> _onMessageArrived() async {
    if (!_isAppInBackground) {
      return;
    }

    if (ZegoZIMKitNotificationManager.instance.supportBackgroundMessage) {
      final messages = ZIMKitCore.instance.db.messageArrivedNotifier.value;
      ZegoZIMKitNotificationManager.instance
          .showInvitationNotification(messages);
    }
  }
}
